# Scicolpack
Description: Scicolpack is is the Scilab interface to CSCsw/ColPack (https://github.com/CSCsw/ColPack), a Graph Coloring Algorithm Package applied to efficient computation of sparse Jacobian and Hessian. Suppose you have to compute the Jacobian of a function $`f:\mathbb{R}^n\rightarrow \mathbb{R}^m`$ or the Hessian of a function knowing its gradient $`g:\mathbb{R}^n\rightarrow \mathbb{R}^n`$. Once you know its sparsity pattern, even if you have derived it symbolically, it may still be faster to estimate it by using the techniques which are used by Colpack. Roughly speaking, this works by building a graph where each vertex is associated to a column of the Jacobian or Hessian, and an edge connects two vertices/columns if they are not structurally orthogonal, i.e. have at least one non-zero term in a common row. Then a proper coloring is done on this graph: at least, two adjacent vertices cannot have the same color, but more properties of the coloring may be expected. The coloring defines a partition of the columns under the form of p subsets and the Jacobian (resp. Hessian) can be recovered from only p evaluation of directional derivatives of $`f`$ (resp. $`g`$). For example, for a tridiagonal matrix the value of p is 3. In the Scilab interface these directional derivatives are approximated by using finite differences (the toolbox allows to compute them by using the complex step technique up to machine precision). See the following example of a sparse 219x85 Jacobian which needs only 4 colors, hence 4 evaluations of directional derivative to be approximated:
 <p align="center">
  <img src="images/HBJac.png" />
</p>

# How to install
From Scilab command line:
````
atomsInstall scicolpack
````
or from Atoms GUI, in the Numerical Maths section.
# How to build (needs a development version of Scilab)
First get the sources of the toolbox
````
git clone https://gitlab.com/mottelet/scicolpack.git
````
then from Scilab, execute the `builder.sce` script in the `scicolpack folder`. Under Linux or macOS, the first time you build, ColPack (https://github.com/CSCsw/ColPack) will be dowloaded from its repository and built with default options (see the script `build.sh` in `thirdparty/build`). 

# Example
Here are the timings for the computation of a minimal surface defined as the graph of $`(x,y)\rightarrow f(x,y)`$ on $`[-1,1] \times [-1,1]`$ discretized on a 51x51 grid (operators are approximated with order two finite differences). At each step of the Newton algorithm the Jacobian of the non-linear system $`F(f)=0`$  has only 22801 non-zero terms (0.3% of the 6765201 terms of full matrix) and is estimated by only 16 evaluations of $`F`$. Resolution takes only 0.119 seconds and as a comparison the resolution of the same system with legacy tools of Scilab is more than 2000 times slower.
<p align="center">
  <img src="images/minsurface.png" />
</p>
