//
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) 2022 - Stephane MOTTELET
//
// This file is hereby licensed under the terms of the GNU GPL v2.0,
// For more information, see the COPYING file which you should have received
//
//--------------------------------------------------------------------------

path_builder = get_absolute_file_path("builder_gateway_cpp.sce");
path_root = fullpath(fullfile(path_builder, "..", ".."));

gateway_files = [
    "sci_spCompHessian.cpp"
    "sci_spCompJacobian.cpp"
    "sci_percent_spCompGeneric_e.cpp"
    "sci_percent_spCompGeneric_clear.cpp"];

files_in_src = [
    "spCompGeneric.cpp"
    "spCompHessian.cpp"
    "spCompJacobian.cpp"
];

for i=1:size(files_in_src,"*")
    copyfile(fullfile(path_root,"src","cpp",files_in_src(i)),fullfile(path_builder,files_in_src(i)));
end

files_to_compile = [
    files_in_src
    gateway_files
]

tools_path  = fullfile(path_root, "thirdparty", getos());
include_ColPack = fullfile(tools_path, "include", "ColPack");
cflags = " -I" + fullfile(path_root, "includes");
cflags = cflags + " -I" + include_ColPack;

if getos() == "Windows" then
    cflags = cflags + " -I" + fullfile(SCI,"libs", "Eigen", "includes");
    ldflags = fullpath(fullfile(tools_path, "lib", "colPack.lib"));
else
    buildlibdir = fullfile(tools_path,"lib");
    loadlibdir = fullfile("..","..","thirdparty",getos(),"lib")
    ldflags  = "-L" + buildlibdir + " -lColPack"
    if getos() == "Darwin" 
        if getenv("CONDA_PREFIX","UNDEFINED") <> "UNDEFINED"
           cflags = cflags + " -I" + fullfile(getenv("CONDA_PREFIX"),"include","eigen3");
        else
           cflags = cflags + " -I" + fullfile(SCI,"lib", "Eigen", "includes");   
        end
         ldflags = ldflags + " -Wl,-rpath,@loader_path/"+loadlibdir;  
    else
        cflags = cflags + " -I" + fullfile(SCI,"lib", "Eigen", "include"); 
        ldflags = ldflags + " -Wl,-rpath,''\$\$ORIGIN/"+loadlibdir+"''";  
    end
end

libs = [];
ilib_verbose(2)

names = ["%_spCompJacobian_e", "sci_percent_spCompGeneric_e", "cppsci";
         "%_spCompJacobian_clear", "sci_percent_spCompGeneric_clear", "cppsci";
         "%_spCompHessian_e", "sci_percent_spCompGeneric_e", "cppsci";
         "%_spCompHessian_clear", "sci_percent_spCompGeneric_clear", "cppsci";
         "spCompHessian", "sci_spCompHessian", "cppsciopt";
         "spCompJacobian", "sci_spCompJacobian", "cppsciopt"];

tbx_build_gateway("scicolpack", names, files_to_compile, path_builder, libs, ldflags, cflags);

for i=1:size(files_in_src,"*")
    deletefile(fullfile(path_builder,files_in_src(i)));
end
